# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.WalletSummaryOptions import WalletSummaryOptions


class WalletSummary:
    paid: int
    free: int
    total: int

    def __init__(
        self,
        paid: int,
        free: int,
        total: int,
        options: Optional[WalletSummaryOptions] = WalletSummaryOptions(),
    ):
        self.paid = paid
        self.free = free
        self.total = total

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.paid is not None:
            properties["paid"] = self.paid
        if self.free is not None:
            properties["free"] = self.free
        if self.total is not None:
            properties["total"] = self.total

        return properties
